#include "Player.h"
#include <d3dx9math.h>
//#include "Controller.h"
struct Vector
{
	float x,y,z;
};

#define PI 3.14159

Player::Player(float x,float y,float z,float lX,float lY,float lZ)
{
	
	this->x = x;
	this->y = y;
	this->z = z;
	this->lookX = lX;
	this->lookY = lY;
	this->lookZ = lZ;	
}
float Player::getLX()
{
	return sin(this->lookY) + x;	
}
float Player::getLY()
{	
	return tan(this->lookX) + y;
}
float Player::getLZ()
{
	return cos(this->lookY) + z;
}
Player::Player(float x,float y,float z)
{
	
	this->x = x;
	this->y = y;
	this->z = z;
	this->lookX = 0;
	this->lookY = 0;
	this->lookZ = 0;
}
float Player::getX()
{
	return this->x;
}
float Player::getY()
{
	return this->y;
}
float Player::getZ()
{
	return this->z;
}
D3DXMATRIXA16 Player::getLookPt()
{
	float xl;
	float yl; // what we're looking at
	float zl;

	float ux;
	float uy; // up-looking points
	float uz;
	xl = sin(this->lookY) + x;
	zl = cos(this->lookY) + z;
	yl = tan(this->lookX) + y;

	ux = 0;
	uy = 1;
	uz = 0;
	D3DXVECTOR3 look(xl,yl,zl);
	D3DXVECTOR3 point(this->x,this->y,this->z);
	D3DXVECTOR3 up(ux,uy,uz);

	D3DXMATRIXA16 myMat;
	D3DXMatrixLookAtLH( &myMat, &point, &look, &up );
	return myMat;	
}
void Player::look(float x,float y,float z)
{
	if(this->lookX + x < PI/2 && this->lookX + x > -PI/2)
	{
	this->lookX += x;
	}
	
	this->lookY += y;
	this->lookZ += z;
}
void Player::move(float x,float y,float z)
{
	this->x += x;
	this->y += y;
	this->z += z;
}

void UnitVectorize(Vector* what)
{
	float total = sqrt(what->x*what->x + what->y*what->y + what->z*what->z);
	what->x /= total;
	what->y /= total;
	what->z /= total;

}
Vector vectorAdd(Vector* one,Vector* two)
{
	Vector answer;
	answer.x = one->x + two->x;
	answer.y = one->y + two->y;
	answer.z = one->z + two->z;
	return answer;
}
Vector vectorMul(Vector* one,float scalar)
{
	Vector answer;
	answer.x = one->x * scalar;
	answer.y = one->y * scalar;
	answer.z = one->z * scalar;
	return answer;
}
void Player::Run()
{	
	if(myController != NULL)
	{
		//this->myController->Run();
	}
}
void Player::setController(Controller* what)
{
	this->myController = what;
}
void Player::moveRel(float x,float y,float z)
{
	Vector straight;
	Vector rightAngle;
	Vector totalMove;
	float xl;
	float yl; // what we're looking at
	float zl;
	
	xl = sin(this->lookY);
	zl = cos(this->lookY);
	yl = tan(this->lookX);

	
	straight.x = xl;
	straight.y = yl; // this is now our straight ahead vector
	straight.z = zl;
	
	rightAngle.x= -zl;
	rightAngle.y = 0; // this is now our 90-degree vector
	rightAngle.z = xl;	

	UnitVectorize(&straight);
	UnitVectorize(&rightAngle);
	
	rightAngle = vectorMul(&rightAngle,x); // rightAngle now has the direction and magnitude of the desired direction
	straight = vectorMul(&straight,z);

	totalMove = vectorAdd(&rightAngle,&straight);

	this->move(totalMove.x,totalMove.y,totalMove.z);
}

