#include "nGon.h"
#include "D3Drunner.h"
#include <d3d9.h>
#include <math.h>
#include <stdio.h>
#include <d3dx9.h>
void writeCrap(const char * what);
void writeCrap(int what);
void writeCrap(float what);
void writeCrap(float what);

#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL)

nGon::nGon(int triangles, D3Drunner* myd3d)
{
	HRESULT hr;
	LPDIRECT3DDEVICE9 device;

	device = myd3d->getDevice();
	
	hr = device->CreateVertexBuffer(triangles * sizeof(TRIANGLE),0,D3DFVF_CUSTOMVERTEX,D3DPOOL_DEFAULT,&myVB,NULL);

	if(FAILED(hr))
	{
		writeCrap("Could not create vertex buffer");
	}

	this->mtrl.Diffuse.r = this->mtrl.Ambient.r = 1.0f;
	this->mtrl.Diffuse.g = this->mtrl.Ambient.g = 1.0f;
	this->mtrl.Diffuse.b = this->mtrl.Ambient.b = 0;
	this->mtrl.Diffuse.a = this->mtrl.Ambient.a = 1.0f;

	this->curTri=0;
	this->maxTri=triangles;
	this->whereVertex=NULL;
	this->avgValid =false;
}

MyVector nGon::cross(MyVector u,MyVector v)
{
	MyVector normal;
	normal.x = u.y * v.z - u.z * v.y;
	normal.y = u.z * v.x - u.x * v.z;
	normal.z = u.x * v.y - u.y * v.x;
	return normal;
}

void nGon::addTri(TRIANGLE * newTri)
{
	HRESULT hr;
	void* whereWrite;
	if(curTri < maxTri)
	{
#ifdef USING_NORMAL	
		MyVector normal;
		
		for(int x=0;x<3;x++)
		{

			MyVector u,v;
			u.x = newTri->v[(x+1)%3].position.x - newTri->v[x].position.x;
			u.y = newTri->v[(x+1)%3].position.y - newTri->v[x].position.y;
			u.z = newTri->v[(x+1)%3].position.z - newTri->v[x].position.z;

			v.x = newTri->v[(x+2)%3].position.x - newTri->v[x].position.x;
			v.y = newTri->v[(x+2)%3].position.y - newTri->v[x].position.y;
			v.z = newTri->v[(x+2)%3].position.z - newTri->v[x].position.z;

			

			normal = cross(u,v);
			newTri->v[x].normal.x = normal.x;
			newTri->v[x].normal.y = normal.x;
			newTri->v[x].normal.z = normal.x;

		}
#endif
		hr = this->myVB->Lock(curTri * sizeof(TRIANGLE),sizeof(TRIANGLE),&whereWrite,0);
		if(FAILED(hr))
		{
			writeCrap("Failure to lock for adding triangle");
			return;
		}
		if(curTri==0)
		{
			whereVertex = (CUSTOMVERTEX*)whereWrite;
		}
		memcpy(whereWrite,newTri,sizeof(TRIANGLE));
		hr = this->myVB->Unlock();
		curTri++;
		avgValid = false;
		

		
		//sprintf(myText,"v1 = (%f,%f,%f) v2 = (%f,%f,%f) v3 = (%f,%f,%f)",newTri->v[0].x,newTri->v[0].y,newTri->v[0].z,newTri->v[1].x,newTri->v[1].y,newTri->v[1].z,newTri->v[2].x,newTri->v[2].y,newTri->v[2].z);
		
	}
	else
	{
		writeCrap("Too many triangles");
	}
}
void nGon::scale(float scalar)
{
	calcAvg();
	for(int x=0;x<curTri*3;x++)
	{
		this->whereVertex[x].position.x = ((whereVertex[x].position.x - midX) * scalar) + midX;
		this->whereVertex[x].position.y = ((whereVertex[x].position.y - midY) * scalar) + midY;
		this->whereVertex[x].position.z = ((whereVertex[x].position.z - midZ) * scalar) + midZ;
	}
}
void nGon::Transform(float dx ,float dy,float dz)
{
	for(int x=0;x<curTri*3;x++)
	{
		whereVertex[x].position.x += dx;
		whereVertex[x].position.y += dy;
		whereVertex[x].position.z += dz;
	
	}
	this->midX += dx;
	this->midY += dy;
	this->midZ += dz;
}
void nGon::setVertex(CUSTOMVERTEX* what,int index)
{
	if(index < (this->maxTri * 3))
	{
		void* whereWrite;
		HRESULT hr;
		hr = this->myVB->Lock(index * sizeof(CUSTOMVERTEX),sizeof(CUSTOMVERTEX),&whereWrite,0);
		if(FAILED(hr))
		{
			writeCrap("Could not lock");
		}
		memcpy(whereWrite,what,sizeof(CUSTOMVERTEX));
		this->myVB->Unlock();
	}

}



void nGon::Render(LPDIRECT3DDEVICE9 device,App* myApp,int pass)
{
	if(pass == 1)
	{	
		//device->SetMaterial(&this->mtrl);
		device->SetStreamSource(0,this->myVB,0,sizeof(CUSTOMVERTEX));
		device->SetFVF(D3DFVF_CUSTOMVERTEX);
		device->DrawPrimitive(D3DPT_TRIANGLELIST, 0, this->curTri);
	}
}
void nGon::rotateY(float theta,float x,float z)
{
	float x0;
	float z0;
	for(int i=0;i<curTri*3;i++)
	{
		x0 = whereVertex[i].position.x - x;
		z0 = whereVertex[i].position.z - z;
		whereVertex[i].position.x = (x0 * cos(theta) - z0 * sin(theta)) + x;
		whereVertex[i].position.z = (x0 * sin(theta) + z0 * cos(theta)) + z;

		
	}
	avgValid = false;
}
void nGon::calcAvg()
{
	if(!avgValid)
	{
		float xtot =0;
		float ytot =0;
		float ztot =0;
		long counter =0;
		for(int i=0;i<curTri*3;i++)
		{
			xtot += whereVertex[i].position.x;
			ytot += whereVertex[i].position.y;
			ztot += whereVertex[i].position.z;
			counter++;
		}
		midX = xtot / counter;
		midY = ytot / counter;
		midZ = ztot / counter;
	}
	avgValid = true;
}
void nGon::rotateY(float theta)
{
	calcAvg();
	rotateY(theta,this->midX,this->midZ);
	avgValid = false;
}
int nGon::getNumTri()
{
	return this->curTri;
}
LPDIRECT3DVERTEXBUFFER9 nGon::getVB()
{
	return this->myVB;
}